@@ -7,7 +7,7 @@
 #define XXXXXXX    0
 #define USE_PENALTY_EX  1
 #define FASTMATCHCALC 1
-#define MCD 0
+#define SLOW 0
 
 
 static TLS double **impmtx = NULL;
@@ -19,7 +19,7 @@ double imp_match_out_sc( int i1, int j1 )
 	return( impmtx[i1][j1] );
 }
 
-#if 0
+#if 1
 static void imp_match_out_vead_gapmap( double *imp, int i1, int lgth2, int *gapmap2 )
 {
 #if FASTMATCHCALC
@@ -36,7 +36,6 @@ static void imp_match_out_vead_gapmap( double *imp, int i1, int lgth2, int *gapm
 }
 
 
-
 static void imp_match_out_vead_tate_gapmap( double *imp, int j1, int lgth1, int *gapmap1 )
 {
 #if FASTMATCHCALC
@@ -64,8 +63,6 @@ static void imp_match_out_vead( double *imp, int i1, int lgth2 )
 		*imp++ += pt[j];
 #endif
 }
-
-
 static void imp_match_out_vead_tate( double *imp, int j1, int lgth1 )
 {
 	int i;
@@ -462,23 +459,65 @@ for( i = 0; i<impalloclen; i++ )
 }
 #endif
 
-#if MCD
-static void match_calc_dynamicmtx( double **pairoffset, double **n_dynamicmtx, double *match, int n1, char **seq1, double *eff1, int n2, char **seq2, double *eff2, int i1, int lgth2, double **doublework, int **intwork, int initialize, int flip ) 
+static void match_calc_del( int **which, double ***matrices, double *match, int n1, char **seq1, double *eff1, int n2, char **seq2, double *eff2, int i1, int lgth2, int mid, int nmask, int *mask1, int *mask2 ) 
+{
+// osoi!
+	int i, j, k, m;
+	int c1, c2;
+//	fprintf( stderr, "\nmatch_calc_dynamicmtx... %d", i1 );
+//	fprintf( stderr, "\nseq1[0]=%s\n", seq1[0] );
+//	fprintf( stderr, "\nseq2[0]=%s\n", seq2[0] );
+//	for( i=0; i<n1; i++ ) for( j=0; j<n2; j++ )
+//	{
+//		if( flip ) reporterr( "in match_calc_slow, which[%d][%d] = %d\n", j, i, which[j][i] );
+//		else       reporterr( "in match_calc_slow, which[%d][%d] = %d\n", i, j, which[i][j] );
+//	}
+	for( k=0; k<lgth2; k++ )
+	{
+		for( m=0; m<nmask; m++ )
+		{
+			i = mask1[m];
+			j = mask2[m];
+//			reporterr( "Deleting %d-%d (c=%d)\n", i, j, mid );
+//			if( k==0 ) fprintf( stderr, "pairoffset[%d][%d] = %f\n", i, j, po );
+			c1 = amino_n[(int)seq1[i][i1]];
+			c2 = amino_n[(int)seq2[j][k]];
+//			reporterr( "k=%d, c1=%d, c2=%d, seq1[i][i1]=%c, seq2[%d][%d]=%c\n", k, c1, c2, seq1[i][i1], j, k, seq2[j][k] );
+			if( seq1[i][i1] == '-' || seq2[j][k] == '-' ) continue;
+			if( c1 < 0 || c2 < 0 ) continue;
+//			fprintf( stderr, "c1=%d, c2=%d\n", c1, c2 );
+//			fprintf( stderr, "match[k] = %f -> ", match[k], mid );
+			match[k] -= matrices[mid][c1][c2] * eff1[i] * eff2[j];
+//			fprintf( stderr, "match[k] = %f (mid=%d)\n", match[k], mid );
+		}
+	}
+//	fprintf( stderr, "done\n" );
+	return;
+}
+
+
+#if SLOW
+static void match_calc_slow( int **which, double ***matrices, double *match, int n1, char **seq1, double *eff1, int n2, char **seq2, double *eff2, int i1, int lgth2, double **doublework, int **intwork, int initialize, int flip ) 
 {
 // osoi!
 	int i, j, k;
 	int c1, c2;
-	double po;
+	int mid;
 //	fprintf( stderr, "\nmatch_calc_dynamicmtx... %d", i1 );
 //	fprintf( stderr, "\nseq1[0]=%s\n", seq1[0] );
 //	fprintf( stderr, "\nseq2[0]=%s\n", seq2[0] );
+//	for( i=0; i<n1; i++ ) for( j=0; j<n2; j++ )
+//	{
+//		if( flip ) reporterr( "in match_calc_slow, which[%d][%d] = %d\n", j, i, which[j][i] );
+//		else       reporterr( "in match_calc_slow, which[%d][%d] = %d\n", i, j, which[i][j] );
+//	}
 	for( k=0; k<lgth2; k++ )
 	{
 		match[k] = 0.0;
 		for( i=0; i<n1; i++ ) for( j=0; j<n2; j++ )
 		{
-			if( flip ) po = pairoffset[j][i];
-			else       po = pairoffset[i][j];
+			if( flip ) mid = which[j][i];
+			else       mid = which[i][j];
 //			if( k==0 ) fprintf( stderr, "pairoffset[%d][%d] = %f\n", i, j, po );
 			c1 = amino_n[(int)seq1[i][i1]];
 			c2 = amino_n[(int)seq2[j][k]];
@@ -486,11 +525,10 @@ static void match_calc_dynamicmtx( double **pairoffset, double **n_dynamicmtx, d
 			if( c1 < 0 || c2 < 0 ) continue;
 //			fprintf( stderr, "c1=%d, c2=%d\n", c1, c2 );
 			if( flip ) 
-				match[k] += ( n_dynamicmtx[c1][c2] + po * 600 ) * eff1[i] * eff2[j];
+				match[k] += matrices[mid][c1][c2] * eff1[i] * eff2[j];
 			else
-				match[k] += ( n_dynamicmtx[c1][c2] + po * 600 ) * eff1[i] * eff2[j];
-//			fprintf( stderr, "match[k] = %f\n", match[k] );
-//			fprintf( stderr, "pairoffset[i][j] = %f\n", pairoffset[i][j] );
+				match[k] += matrices[mid][c1][c2] * eff1[i] * eff2[j];
+//			fprintf( stderr, "match[k] = %f (which=%d)\n", match[k], mid );
 		}
 	}
 //	fprintf( stderr, "done\n" );
@@ -1739,10 +1777,11 @@ double A__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, in
 }
 
 
-double A__align_variousdist( double **pairoffset, double ***matrices, double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, double **eff1s, double **eff2s, int icyc, int jcyc, int alloclen, LocalHom ***localhom, double *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
+double A__align_variousdist( int **which, double ***matrices, double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, double **eff1s, double **eff2s, int icyc, int jcyc, int alloclen, LocalHom ***localhom, double *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 
+
 //	int k;
 	register int i, j, c;
 	int lasti, lastj;      /* outgap == 0 -> lgth1, outgap == 1 -> lgth1+1 */
@@ -1808,6 +1847,7 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 	double curm = 0.0;
 	double *wmrecordspt, *wmrecords1pt, *prevwmrecordspt;
 	int *warpipt, *warpjpt;
+	int *nmask, **masklist1, **masklist2;
 
 
 	if( seq1 == NULL )
@@ -1857,10 +1897,35 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 	}
 
 
+#if SLOW
+	nmask = calloc( maxdistclass, sizeof( int ) );
+#else
+	masklist1 = AllocateIntMtx( maxdistclass, 0 );
+	masklist2 = AllocateIntMtx( maxdistclass, 0 );
+	nmask = calloc( maxdistclass, sizeof( int ) );
 
-//	fprintf( stderr, "\npairoffset = \n" );
-//	for( i=0; i<icyc; i ++ ) for( j=0; j<jcyc; j ++ )
-//		fprintf( stderr, "%d-%d, %f\n", i, j, pairoffset[i][j] );
+	for( c=0; c<maxdistclass; c++ )
+	{
+		for( i=0; i<icyc; i++ ) for( j=0; j<jcyc; j++ )
+		{
+			if( eff1s[c][i] * eff2s[c][j] != 0.0 )
+			{
+
+				if( c != which[i][j] )
+				{
+					masklist1[c] = realloc( masklist1[c], sizeof( int ) * nmask[c]+1 );
+					masklist2[c] = realloc( masklist2[c], sizeof( int ) * nmask[c]+1 );
+
+					masklist1[c][nmask[c]] = i;
+					masklist2[c][nmask[c]] = j;
+					nmask[c]++;
+				}
+			}
+		}
+	}
+	for( c=0; c<maxdistclass; c++ ) if( nmask[c] ) break;
+	if( c<maxdistclass ) reporterr( "Found a complex grouping. This step may be a bit slow.\n", c );
+#endif
 
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
@@ -2065,14 +2130,16 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 	}
 #endif
 
+#if SLOW
+#else
 //	cpmx_calc_new( seq1, cpmx1, eff1, lgth1, icyc );
 //	cpmx_calc_new( seq2, cpmx2, eff2, lgth2, jcyc );
 	for( c=0; c<maxdistclass; c++ )
 	{
-//		fprintf( stderr, "c=%d\n", c );
 		cpmx_calc_new( seq1, cpmx1s[c], eff1s[c], lgth1, icyc );
 		cpmx_calc_new( seq2, cpmx2s[c], eff2s[c], lgth2, jcyc );
 	}
+#endif
 
 	if( sgap1 )
 	{
@@ -2147,9 +2214,8 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 //	for( i=0; i<icyc; i++ ) fprintf( stderr, "seq1[i] = %s\n", seq1[i] );
 //	for( j=0; j<jcyc; j++ ) fprintf( stderr, "seq2[j] = %s\n", seq2[j] );
 
-#if MCD
-	match_calc_dynamicmtx( pairoffset, n_dynamicmtx, initverticalw, jcyc, seq2, eff2, icyc, seq1, eff1, 0, lgth1, *doublework, *intwork, 1, 1 );
-//	for( i=0; i<lgth1; i++ ) fprintf( stderr, "%d - %f\n", i, initverticalw[i] );
+#if SLOW
+	match_calc_slow( which, matrices, initverticalw, jcyc, seq2, eff2, icyc, seq1, eff1, 0, lgth1, *doublework, *intwork, 1, 1 );
 #else
 	fillzero( initverticalw, lgth1 );
 	for( c=0; c<maxdistclass; c++ )
@@ -2159,6 +2225,7 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 //		for( i=0; i<lgth2; i++ ) fprintf( stderr, "seq2[i] = %c, cpmx2s[c][3][%d] = %f\n", seq2[0][i], i, cpmx2s[c][3][i] );
 		match_calc_add( matrices[c], initverticalw, cpmx2s[c], cpmx1s[c], 0, lgth1, doublework[c], intwork[c], 1 );
 //		for( i=0; i<lgth1; i++ ) fprintf( stderr, "c=%d, %d - %f\n", c, i, initverticalw[i] );
+		if( nmask[c] ) match_calc_del( which, matrices, initverticalw, jcyc, seq2, eff2, icyc, seq1, eff1, 0, lgth1, c, nmask[c], masklist2[c], masklist1[c] );
 	}
 //	for( i=0; i<lgth1; i++ ) fprintf( stderr, "%d - %f\n", i, initverticalw[i] );
 #endif
@@ -2168,14 +2235,17 @@ double A__align_variousdist( double **pairoffset, double ***matrices, double **n
 	if( localhom )
 		imp_match_out_vead_tate( initverticalw, 0, lgth1 ); // 060306
 
-#if MCD
-	match_calc_dynamicmtx( pairoffset, n_dynamicmtx, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, 0, lgth2, *doublework, *intwork, 1, 0 );
+#if SLOW
+	match_calc_slow( which, matrices, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, 0, lgth2, *doublework, *intwork, 1, 0 );
 //	for( i=0; i<lgth2; i++ ) fprintf( stderr, "%d - %f\n", i, currentw[i] );
 //	exit( 1 );
 #else
 	fillzero( currentw, lgth2 );
 	for( c=0; c<maxdistclass; c++ )
+	{
 		match_calc_add( matrices[c], currentw, cpmx1s[c], cpmx2s[c], 0, lgth2, doublework[c], intwork[c], 1 );
+		if( nmask[c] ) match_calc_del( which, matrices, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, 0, lgth2, c, nmask[c], masklist1[c], masklist2[c] );
+	}
 //	for( i=0; i<lgth2; i++ ) fprintf( stderr, "%d - %f\n", i, currentw[i] );
 //	exit( 1 );
 #endif
@@ -2264,6 +2334,9 @@ for( i=0; i<lgth2; i++ )
 		if( chudanpt && *chudanpt != chudanref ) 
 		{
 //			fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
+			if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
+			if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+			if( nmask ) free( nmask ); nmask = NULL;
 			*chudanres = 1;
 			return( -1.0 );
 		}
@@ -2274,12 +2347,15 @@ for( i=0; i<lgth2; i++ )
 
 		previousw[0] = initverticalw[i-1];
 
-#if MCD
-		match_calc_dynamicmtx( pairoffset, n_dynamicmtx, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, i, lgth2, *doublework, *intwork, 0, 0 );
+#if SLOW
+		match_calc_slow( which, matrices, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, i, lgth2, *doublework, *intwork, 0, 0 );
 #else
 		fillzero( currentw, lgth2 );
 		for( c=0; c<maxdistclass; c++ )
+		{
 			match_calc_add( matrices[c], currentw, cpmx1s[c], cpmx2s[c], i, lgth2, doublework[c], intwork[c], 0 );
+			if( nmask[c] ) match_calc_del( which, matrices, currentw, icyc, seq1, eff1, jcyc, seq2, eff2, i, lgth2, c, nmask[c], masklist1[c], masklist2[c] );
+		}
 #endif
 #if 0
 		if( i == 1 )
@@ -2362,6 +2438,9 @@ fprintf( stderr, "\n" );
 			if( chudanpt && *chudanpt != chudanref ) 
 			{
 //				fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
+				if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
+				if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+				if( nmask ) free( nmask ); nmask = NULL;
 				*chudanres = 1;
 				return( -1.0 );
 			}
@@ -2536,5 +2615,10 @@ fprintf( stderr, "\n" );
 
 //	fprintf( stderr, "wm = %f\n", wm );
 
+
+	if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
+	if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+	if( nmask ) free( nmask ); nmask = NULL;
+
 	return( wm );
 }
