@@ -767,8 +767,26 @@ void constants( int nseq, char **seq )
 		double *freq1;
 		double *datafreq;
 		double average;
+		double iaverage;
 //		double tmp;
 		double **n_distmp;
+		int makeaverage0;
+
+
+		if( nblosum == 0 )
+		{
+			reporterr( "nblosum=%d??\n", nblosum );
+			exit( 1 );
+		}
+		if( nblosum < 0 )
+		{
+			nblosum *= -1;
+			makeaverage0 = 0;
+		}
+		else
+		{
+			makeaverage0 = 1;
+		}
 
 		nalphabets = 26;
 		nscoredalphabets = 20;
@@ -850,8 +868,11 @@ void constants( int nseq, char **seq )
 		fprintf( stdout, "####### average2  = %f\n", average );
 #endif
 
-		for( i=0; i<20; i++ ) for( j=0; j<20; j++ ) 
-			n_distmp[i][j] -= average;
+		if( makeaverage0 )
+		{
+			for( i=0; i<20; i++ ) for( j=0; j<20; j++ ) 
+				n_distmp[i][j] -= average;
+		}
 #if TEST
 		fprintf( stdout, "average2 = %f\n", average );
 		fprintf( stdout, "after average substruction : \n" );
@@ -925,12 +946,12 @@ void constants( int nseq, char **seq )
 			average = 0.0;
         	for( i=0; i<20; i++ ) for( j=0; j<20; j++ )
 				average += n_distmp[i][j] * freq1[i] * freq1[j];
-			fprintf( stdout, "average = %f\n", average );
+			fprintf( stdout, "\naverage = %f\n", average );
 
-			average = 0.0;
+			iaverage = 0.0;
         	for( i=0; i<20; i++ )
-				average += n_distmp[i][i] * freq1[i];
-			fprintf( stdout, "itch average = %f\n", average );
+				iaverage += n_distmp[i][i] * freq1[i];
+			fprintf( stdout, "itch average = %f, E=%f\n", iaverage, average/iaverage );
 			reporterr(       "parameters: %d, %d, %d\n", penalty, penalty_ex, offset );
 
 			
@@ -962,8 +983,10 @@ void constants( int nseq, char **seq )
 		double *mutab;
 		double *datafreq;
 		double average;
+		double iaverage;
 		double tmp;
 		double delta;
+		int makeaverage0;
 
 		nalphabets = 26;
 		nscoredalphabets = 20;
@@ -986,6 +1009,21 @@ void constants( int nseq, char **seq )
 		if( pamN == NOTSPECIFIED )    pamN    = DEFAULTPAMN;
 		if( kimuraR == NOTSPECIFIED ) kimuraR = 1;
 
+		if( pamN == 0 )
+		{
+			reporterr( "pamN=%d??\n", pamN );
+			exit( 1 );
+		}
+		if( pamN < 0 )
+		{
+			pamN *= -1;
+			makeaverage0 = 0;
+		}
+		else
+		{
+			makeaverage0 = 1;
+		}
+
 		penalty = (int)( 600.0 / 1000.0 * ppenalty + 0.5 );
 		penalty_dist = (int)( 600.0 / 1000.0 * ppenalty_dist + 0.5 );
 		penalty_shift = (int)( penalty_shift_factor * penalty );
@@ -1140,8 +1178,11 @@ void constants( int nseq, char **seq )
 		fprintf( stdout, "####### average2  = %f\n", average );
 #endif
 
-		for( i=0; i<20; i++ ) for( j=0; j<20; j++ ) 
-			pamx[i][j] -= average;
+		if( makeaverage0 )
+		{
+			for( i=0; i<20; i++ ) for( j=0; j<20; j++ ) 
+				pamx[i][j] -= average;
+		}
 #if TEST
 		fprintf( stdout, "average2 = %f\n", average );
 		fprintf( stdout, "after average substruction : \n" );
@@ -1228,12 +1269,12 @@ void constants( int nseq, char **seq )
 			average = 0.0;
         	for( i=0; i<20; i++ ) for( j=0; j<20; j++ )
 				average += pamx[i][j] * freq1[i] * freq1[j];
-			fprintf( stdout, "average = %f\n", average );
+			fprintf( stdout, "\naverage = %f\n", average );
 
-			average = 0.0;
+			iaverage = 0.0;
         	for( i=0; i<20; i++ )
-				average += pamx[i][i] * freq1[i];
-			fprintf( stdout, "itch average = %f\n", average );
+				iaverage += pamx[i][i] * freq1[i];
+			fprintf( stdout, "itch average = %f, E=%f\n", average, average/iaverage );
 			reporterr(       "parameters: %d, %d, %d\n", penalty, penalty_ex, offset );
 
 			
