@@ -332,6 +332,9 @@ void arguments( int argc, char *argv[] )
 				case 'A':
 					alg = 'A';
 					break;
+				case '@':
+					alg = 'd';
+					break;
 				case 'N':
 					nevermemsave = 1;
 					break;
@@ -371,9 +374,11 @@ void arguments( int argc, char *argv[] )
 				case 'v':
 					tbrweight = 3;
 					break;
+#if 1
 				case 'd':
 					disp = 1;
 					break;
+#endif
 #if 1
 				case 'O':
 					outgap = 0;
@@ -956,6 +961,7 @@ static void *treebasethread( void *arg )
 			Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 			Falign_udpari_long( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
 			A__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			D__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 			G__align11( NULL, NULL, NULL, 0, 0, 0 ); // iru?
 			free( mseq1 );
 			free( mseq2 );
@@ -1160,6 +1166,18 @@ static void *treebasethread( void *arg )
 //					reporterr(       "%d-%d", clus1, clus2 );
 					pscore = MSalignmm( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
+				case( 'd' ):
+					if( 1 && clus1 == 1 && clus2 == 1 )
+					{
+//						reporterr(       "%d-%d", clus1, clus2 );
+						pscore = G__align11( dynamicmtx, mseq1, mseq2, *alloclen, outgap, outgap );
+					}
+					else
+					{
+//						reporterr(       "%d-%d", clus1, clus2 );
+						pscore = D__align_ls( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					}
+					break;
 				case( 'A' ):
 					if( clus1 == 1 && clus2 == 1 )
 					{
@@ -1389,6 +1407,16 @@ static int treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 		clus2 = fastconjuction_noweight( topol[l][1], aseq, mseq2, effarr2,  indication2 );
 #endif
 
+
+
+
+
+
+
+
+
+
+
 		if( mergeoralign[l] == '1' || mergeoralign[l] == '2' )
 		{
 			newgapstr = "=";
@@ -1530,6 +1558,18 @@ static int treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 //					reporterr(       "%d-%d", clus1, clus2 );
 					pscore = MSalignmm( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
+				case( 'd' ):
+					if( 1 && clus1 == 1 && clus2 == 1 )
+					{
+//						reporterr(       "%d-%d", clus1, clus2 );
+						pscore = G__align11( dynamicmtx, mseq1, mseq2, *alloclen, outgap, outgap );
+					}
+					else
+					{
+//						reporterr(       "%d-%d", clus1, clus2 );
+						pscore = D__align_ls( dynamicmtx, mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+					}
+					break;
 				case( 'A' ):
 					if( clus1 == 1 && clus2 == 1 )
 					{
@@ -1686,6 +1726,7 @@ static int treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 	Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 	Falign_udpari_long( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
 	A__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+	D__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 	G__align11( NULL, NULL, NULL, 0, 0, 0 ); // iru?
 	free( effarr1 );
 	free( effarr2 );
@@ -1704,6 +1745,7 @@ static int treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 	Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 	Falign_udpari_long( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL );
 	A__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+	D__align( NULL,  NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
 	G__align11( NULL, NULL, NULL, 0, 0, 0 ); // iru?
 	if( effarr1 ) free( effarr1 ); effarr1 = NULL;
 	if( effarr2 ) free( effarr2 ); effarr2 = NULL;
@@ -2481,6 +2523,7 @@ int disttbfast( int ngui, int lgui, char **namegui, char **seqgui, int argc, cha
 	
 //			if( !keeplength ) commongappick( njob-nadd, seq );
 			commongappick( njob-nadd, seq );
+
 			for( i=0; i<njob-nadd; i++ ) strcpy( bseq[i], seq[i] );
 
 		}
