@@ -1,6 +1,7 @@
 #include "mltaln.h"
 
 #define DEBUG 0
+#define USEDISTONTREE 1
 
 #if 0
 void mdfymtx( char pair[njob][njob], int s1, double **partialmtx, double **mtx )
@@ -736,7 +737,7 @@ void nodeFromABranch( int nseq, int *result, int **pairwisenode, int ***topol, d
 					
 
 				
-void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pair, int ***topol, int step, int branch, double **smalldistmtx, double **distmtx )
+void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pair, int ***topol, int step, int branch, double **smalldistmtx, double **distmtx, double *distontree )
 {
 	int i, k, j;
 	int r1;
@@ -767,13 +768,28 @@ void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int
 	if( smalldistmtx )
 	{
 		int im, jm;
+#if USEDISTONTREE
+		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
+			smalldistmtx[i][j] = distontree[im] + distontree[jm];
+#else
 		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
-		{
 			smalldistmtx[i][j] = distmtx[MIN(im,jm)][MAX(im,jm)];
-//			fprintf( stderr, "#### %d-%d, %f\n", im, jm, smalldistmtx[i][j] );
-		}
+#endif
+
+#if 0
+		reporterr( "\n" );
+		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
+			reporterr( "smalldistmtx[%d][%d] = %f\n", i, j, smalldistmtx[i][j] );
+
+
+		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
+			smalldistmtx[i][j] = distmtx[MIN(im,jm)][MAX(im,jm)];
+
+		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
+			reporterr( "old smalldistmtx[%d][%d] = %f\n", i, j, smalldistmtx[i][j] );
+if( im > 10 && jm > 10 ) exit( 1 );
+#endif
 	}
-//	free( pair );
 }
 		
 
